import type { RequestHandler } from '@sveltejs/kit';
import { Report, User } from '$lib/server/database';
import * as crypto from 'crypto';
import { env as public_env } from '$env/dynamic/public';
import { env } from '$env/dynamic/private';
import { ReportAddSchema } from './schema';

export const GET = (async ({ url }) => {
	if (env.API_SECRET && url.searchParams.get('secret') !== env.API_SECRET)
		return new Response(null, { status: 401 });

	const user = await User.findOne({ where: { uuid: url.searchParams.get('uuid') ?? '' } });
	if (user === null) return new Response(null, { status: 400 });

	const reports = {
		from_self: await Report.findAll({
			where: { reporter_id: user.id },
			include: [{ model: User, as: 'reported' }]
		}).then((reports) =>
			reports.map((report) => {
				return {
					reported: report.reported
						? {
								username: report.reported.username,
								uuid: report.reported.uuid
							}
						: null,
					subject: report.subject,
					draft: report.draft,
					status: report.status,
					url: `${url.protocol}//${url.host}${public_env.PUBLIC_BASE_PATH || ''}/report/${report.url_hash}`
				};
			})
		),
		to_self: await Report.findAll({
			where: { reported_id: user.id },
			include: [{ model: User, as: 'reporter' }]
		}).then((reports) =>
			reports.map((report) => {
				return {
					reporter: {
						username: report.reporter.username,
						uuid: report.reporter.uuid
					},
					subject: report.subject,
					draft: report.draft,
					status: report.status,
					url: `${url.protocol}//${url.host}${public_env.PUBLIC_BASE_PATH || ''}/report/${report.url_hash}`
				};
			})
		)
	};

	return new Response(JSON.stringify(reports), { status: 200 });
}) satisfies RequestHandler;

export const POST = (async ({ request, url }) => {
	if (env.API_SECRET && url.searchParams.get('secret') !== env.API_SECRET)
		return new Response(null, { status: 401 });

	const parseResult = await ReportAddSchema.safeParseAsync(await request.json());
	if (!parseResult.success) {
		return new Response(null, { status: 400 });
	}
	const data = parseResult.data;

	const reporter = await User.findOne({ where: { uuid: data.reporter } });
	const reported = data.reported
		? await User.findOne({ where: { uuid: data.reported } })
		: undefined;

	if (reporter == null || reported === null) return new Response(null, { status: 400 });

	const report = await Report.create({
		subject: data.reason,
		body: null,
		draft: true,
		status: 'none',
		url_hash: crypto.randomBytes(18).toString('hex'),
		completed: false,
		reporter_id: reporter.id,
		reported_id: reported?.id || null
	});

	return new Response(
		JSON.stringify({
			url: `${url.protocol}//${url.host}${public_env.PUBLIC_BASE_PATH || ''}/report/${
				report.url_hash
			}`
		}),
		{
			status: 201
		}
	);
}) satisfies RequestHandler;
